package com.jiuqi.office.excel;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.apache.poi.hssf.util.HSSFColor;

public class HSSFHelper {
   private static final int[] WIN32_COLORS = new int[]{13158600, 0, 13743257, 14405055, 15790320, 16777215, 6579300, 0, 0, 0, 11842740, 16578548, 11250603, 16750899, 16777215, 15790320, 10526880, 8421504, 0, 5525059, 16777215, 6908265, 14935011, 0, 14811135, 0, 13395456, 15389113, 15918295, 16750899, 15790320, 0};
   private static List colorElements = new ArrayList();
   private static Map colorFinder = new Hashtable();

   private HSSFHelper() {
   }

   public static int pixelToWidth(int value) {
      if (value <= 0) {
         return 0;
      } else if (value == 1) {
         return 19;
      } else {
         int width = 19;
         --value;
         width += value / 7 * 256;
         int delta = value % 7;
         if (delta > 0) {
            width += delta / 2 * 73;
            if (delta % 2 == 1) {
               width += 37;
            }
         }

         return width;
      }
   }

   public static int widthToPixel(short value) {
      return (short)(value / 32);
   }

   public static short pixelToHeight(int value) {
      if (value <= 0) {
         return 0;
      } else {
         int height = 7 + (value - 1) * 15;
         return (short)height;
      }
   }

   public static int heightToPixel(short value) {
      return value / 15;
   }

   public static HSSFColor getApproximateColor(int rgbColor) {
      Integer key = new Integer(rgbColor);
      HSSFColor retColor = (HSSFColor)colorFinder.get(key);
      if (retColor == null) {
         retColor = findApproximateColor(rgbColor);
         if (retColor != null) {
            colorFinder.put(key, retColor);
         }
      }

      return retColor;
   }

   public static int getRGBColor(HSSFColor hssfColor) {
      short[] rgb = hssfColor.getTriplet();
      return rgb[0] << 16 | rgb[1] << 8 | rgb[2];
   }

   public static int getRGBColor(short index) {
      HSSFColor hssfColor = (HSSFColor)HSSFColor.getIndexHash().get(new Integer(index));
      return hssfColor != null ? getRGBColor(hssfColor) : -1;
   }

   public static HSSFColor getApproximateTextColor(int rgbColor) {
      if (rgbColor == -16777211) {
         rgbColor = 0;
      }

      return getApproximateColor(rgbColor);
   }

   public static HSSFColor getApproximateEdgeColor(int rgbColor) {
      if (rgbColor == -16777201) {
         rgbColor = 8421504;
      }

      return getApproximateColor(rgbColor);
   }

   private static void buildColorList() {
      synchronized(colorElements) {
         if (colorElements.size() == 0) {
            Map colorMaps = null;
            colorMaps = HSSFColor.getIndexHash();
            Collection colorList = colorMaps.values();
            Iterator colorItr = colorList.iterator();

            while(colorItr.hasNext()) {
               ColorElement element = new ColorElement();
               element.color = (HSSFColor)colorItr.next();
               short[] rgbVals = element.color.getTriplet();
               element.rgb = rgbToColor(rgbVals[0], rgbVals[1], rgbVals[2]);
               element.hsb = color2HSB(element.rgb);
               colorElements.add(element);
            }
         }

      }
   }

   private static HSSFColor findApproximateColor(int rgbColor) {
      buildColorList();
      HSB Hsb = color2HSB(rgbColor);
      double L = Math.sqrt(20000.0);
      ColorElement flagElement = null;

      for(int i = 0; i < colorElements.size(); ++i) {
         ColorElement elem = (ColorElement)colorElements.get(i);
         if (rgbColor == elem.rgb) {
            return elem.color;
         }

         double Q = Hsb.h - elem.hsb.h;
         if (Q > 180.0) {
            Q = 360.0 - Q;
         }

         double E = Math.sqrt(Math.pow(elem.hsb.s - Hsb.s, 2.0) + Math.pow(elem.hsb.b - Hsb.b * Math.cos(Q * Math.PI / 180.0), 2.0) + Math.pow(Hsb.b * Math.sin(Q * Math.PI / 180.0), 2.0));
         if (L > E) {
            L = E;
            flagElement = elem;
         }
      }

      return flagElement == null ? null : flagElement.color;
   }

   private static int rgbToColor(int r, int g, int b) {
      r <<= 16;
      g <<= 8;
      return r | g | b;
   }

   private static HSB color2HSB(int value) {
      HSB hsb = new HSB();
      RGB rgb = colorToRGB(value);
      double M0 = (double)maxColorValue(rgb) / 2.55;
      hsb.setDefault();
      hsb.b = M0;
      if (hsb.b == 0.0) {
         hsb.s = 0.0;
      } else {
         double M1 = (double)minColorValue(rgb) / 2.55;
         hsb.s = (M0 - M1) * 100.0 / M0;
      }

      hsb.h = RGBToHue(rgb);
      return hsb;
   }

   private static RGB colorToRGB(int value) {
      if (value < 0) {
         value = WIN32_COLORS[value & 255];
      }

      int r = (value & 16711680) >> 16;
      int g = (value & '\uff00') >> 8;
      int b = value & 255;
      return new RGB(r, g, b);
   }

   private static double RGBToHue(RGB rgb) {
      int M0 = minColorValue(rgb);
      int M2 = maxColorValue(rgb);
      if (M0 == M2) {
         return 0.0;
      } else {
         int M1 = centerColorValue(rgb);
         if (M2 == M1) {
            if (rgb.r == rgb.g) {
               return 60.0;
            } else {
               return rgb.g == rgb.b ? 180.0 : 300.0;
            }
         } else {
            double F = (double)(60 * (M1 - M0) / (M2 - M0));
            double H;
            if (rgb.r == M2) {
               if (rgb.g < rgb.b) {
                  H = -F;
               } else {
                  H = F;
               }
            } else if (rgb.g == M2) {
               if (rgb.b < rgb.r) {
                  H = -F;
               } else {
                  H = F;
               }

               H += 120.0;
            } else {
               if (rgb.r < rgb.g) {
                  H = -F;
               } else {
                  H = F;
               }

               H += 240.0;
            }

            if (H < 0.0) {
               H += 360.0;
            }

            return H;
         }
      }
   }

   private static int maxColorValue(RGB rgb) {
      int max = rgb.r;
      if (rgb.g > max) {
         max = rgb.g;
      }

      if (rgb.b > max) {
         max = rgb.b;
      }

      return max;
   }

   private static int minColorValue(RGB rgb) {
      int min = rgb.r;
      if (rgb.g < min) {
         min = rgb.g;
      }

      if (rgb.b < min) {
         min = rgb.b;
      }

      return min;
   }

   private static int centerColorValue(RGB rgb) {
      if (rgb.r < rgb.g && rgb.r < rgb.b) {
         return rgb.g < rgb.b ? rgb.g : rgb.b;
      } else if (rgb.g < rgb.r && rgb.g < rgb.b) {
         return rgb.r < rgb.b ? rgb.r : rgb.b;
      } else {
         return rgb.r < rgb.g ? rgb.r : rgb.g;
      }
   }

   protected static class ColorElement {
      public int rgb;
      public HSB hsb;
      public HSSFColor color;
   }

   protected static class HSB {
      public double h;
      public double s;
      public double b;

      public void setDefault() {
         this.h = 65535.0;
         this.s = 0.0;
         this.b = 0.0;
      }
   }

   private static class RGB {
      public int r;
      public int g;
      public int b;

      public RGB() {
      }

      public RGB(int r, int g, int b) {
         this.r = r;
         this.g = g;
         this.b = b;
      }
   }
}
